char *vcbusMasterTime=
/********************************************************************
 *                                                                  *
 *                        EDSS Software Bus                         *
 */ "EDSS_SB busMasterTime.c 0.7/prt/ncsu.csc/balay/15-Aug-96";/*
 *                                                                  *
 *                     Copyright (c) 1994 by                        *
 *                                                                  *
 *                  North Carolina State University                 *
 *              Department of Computer Science, Box 8206            *
 *                      Raleigh, NC 27695-8206                      *
 *                               and                                *
 *           MCNC, North Carolina Supercomputing Center             *
 *              Environmental Programs, P.O. Box 12889              *
 *                 Research Triangle Park, 27709                    *
 *      Made available under terms of the GNU Public License        *
 *                                                                  *
 ********************************************************************/
 
/********************************************************************
                    About:  EDSS_SB - busMasterTime.c
 
 busMasterTime.c consists of functions for measuring busMaster service
                  time using various system functions :
              gethrtime - high resolution real time
              getrusage - for user and system time used
              times     -     -ditto- 
 
 INPUT FILES:       stdin stream
 OUTPUT FILES:      stdout stream
 ERROR HANDLING:    output to stderr (screen)
 INPUT PARAMETERS:  client name (ascii string), optional
 INPUT OPTIONS:     if client name is not input, default is
 
 NETWORKING:        remote/network connectivity is via bus library (sockets)
 
 COMPILATION:       (see makefile)
 
 KNOWN BUGS:  :-(
 
 OTHER NOTES: :-)
 
 ********************************************************************/
 
/* @(#)busMasterTime.c	1.1 /tmp_mnt/pub/storage/edss/framework/src/bus/bus_src/SCCS/s.busMasterTime.c 15 Aug 1996 16:53:12 */
 
/********************************************************************/
/********************************************************************
                 History: EDSS_SB - busMasterTime.c
 
 Date: 15-Aug-96
 Version: 0.7
 Change Description: ORIGINAL CODE
 Change author: Rajini Balay, NCSU, CSC
 
 ********************************************************************/
#if defined(TIME_BM_RU) || defined(TIME_BM_HR) || defined(TIME_BM_TIMES)

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include        <sys/resource.h>

#include <sys/times.h>
#include <sys/param.h>
 

#ifdef TIME_BM_RU 
static  struct timeval          time_start, time_stop; /* for real time */
static  struct rusage           ru_start, ru_stop, ru_overhead;  /* for user & sys time */
static  double                  start, stop, useconds, overhead;
#endif

#ifdef TIME_BM_HR
static hrtime_t start_time, stop_time, read_time, end_time, service_time;
#endif

#ifdef TIME_BM_TIMES
struct tms ru_start, ru_end, ru_read, ru_serv;
long read_time, serv_time;
#endif

/***** Functions for using getrusage **********/
#ifdef TIME_BM_RU
void t_start() {
    if (getrusage(RUSAGE_SELF, &ru_start) < 0)
                fprintf(stderr, "t_start: getrusage() error");
}


void t_stop() {
    if (getrusage(RUSAGE_SELF, &ru_stop) < 0)
            fprintf(stderr, "t_stop: getrusage() error");
    if (getrusage(RUSAGE_SELF, &ru_overhead) < 0)
            fprintf(stderr, "t_stop: getrusage() error");

}

double t_getrtime() {
}

double t_getutime() {

        start = ((double) ru_start.ru_utime.tv_sec) * 1000000.0
                                + ru_start.ru_utime.tv_usec;
        stop = ((double) ru_stop.ru_utime.tv_sec) * 1000000.0
                                + ru_stop.ru_utime.tv_usec;
        overhead = ((double) ru_overhead.ru_utime.tv_sec) * 1000000.0
                                + ru_overhead.ru_utime.tv_usec;
        useconds = (stop-start); 
        return useconds;
}

double t_getstime() {
}
#endif

/***** Functions for using gethrtime **********/
#ifdef TIME_BM_HR
void t_start() {
    start_time = gethrtime();
}

void t_stop() {
   end_time = gethrtime();
}

double t_getrtime() {
}

double t_getutime() {
    double utime;

    utime = (double)(end_time - start_time);
    return utime;
}

double t_getstime() {
}
#endif

/***** Functions for using times **********/
#ifdef TIME_BM_TIMES
void t_start() {
    times(&ru_start);
}

void t_stop() {
    times(&ru_end);
}

double t_getrtime() {
}

double t_getutime() {

   double utime;
   utime = (double)((((ru_end.tms_utime - ru_start.tms_utime)+
                (ru_end.tms_stime - ru_start.tms_stime))* 1000000)/2147);
   return utime;
}

double t_getstime() {
}
#endif

#endif
