char *vcmasterDB=
/******************************************************************** 
 *                                                                  * 
 *                        EDSS Software Bus                         * 
 */ "EDSS_SB masterDB.c 0.2/prt/ncsu.csc/morrison.vouk/14-Dec-94"; /*
 *                                                                  * 
 *                     Copyright (c) 1994 by                        *
 *                                                                  * 
 *                  North Carolina State University                 * 
 *              Department of Computer Science, Box 8206            * 
 *                      Raleigh, NC 27695-8206                      * 
 *                               and                                *
 *           MCNC, North Carolina Supercomputing Center             *
 *              Environmental Programs, P.O. Box 12889              *
 *                 Research Triangle Park, 27709                    *
 *                                                                  * 
 *      Made available under terms of the GNU Public License        *
 *                                                                  * 
 ********************************************************************/

/* @(#)masterDB.c	1.2 /tmp_mnt/pub/storage/edss/framework/src/bus/bus_src/SCCS/s.masterDB.c 08 Sep 1995 10:58:20" */

/********************************************************************
                    About:  EDSS_SB - masterDB.c

 Routines and structures that handle bus master data-base calls.
 Will have to be updated and/or replaced once EDSS data-base is
 defined.

 KNOWN BUGS:  :-(

 OTHER NOTES: :-)

 ********************************************************************/

/********************************************************************
                 History: EDSS_SB - masterDB.c

 Date: 14-Dec-94
 Version: 0.1
 Change Description: ORIGINAL CODE  
 Change author: Leland Morrison, NCSU, CSC 

 Date: 15-Dec-94
 Version: 0.2
 Change Description: Added some comments and main headers
 Change author: M. Vouk, NCSU, CSC

 ********************************************************************/

#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "busMaster.h"
#include "masterDB.h"
#include "busError.h"

struct BusTypeNode *
BusMasterAddType(struct BusTypeNode *last,
		 const char *name,
		 int id )
{
  struct BusTypeNode *newNode;
  
  newNode = (struct BusTypeNode *)malloc( sizeof( struct BusTypeNode ) );
  if (newNode == NULL)
    return NULL;
  
  newNode->typeId = id;
  
  newNode->typeName = (char *)malloc( strlen( name ) + 1 );
  if (newNode->typeName == NULL)
    {
      free(newNode);
      return NULL;
    }
  strcpy(newNode->typeName, name);
  
  if(last!=NULL)
    {
      newNode->next = last->next;
      last->next    = newNode;
    }
  else
      newNode->next = newNode;
  return newNode;
}

struct BusTypeNode *
BusMasterFindTypeById(struct BusTypeNode *last,
		      int id )
{
  struct BusTypeNode *current;
  
  current = last->next;
  while( (current != last) && (current->typeId != id) )
      current = current->next;
  
  if (current->typeId == id)
    return current;
  else
    return NULL;
}

struct BusTypeNode *
BusMasterFindTypeByName(struct BusTypeNode *last,
			   const char *name )
{
  struct BusTypeNode *current;
  
  current=last->next;
  while( (current!=last) && strcmp( name, current->typeName ) ) {
      current = current->next;
  }
  
  if (strcmp( name,current->typeName ) == 0) {
      return current;
  }
  else
    return NULL;
}


struct BusModuleNode *
BusMasterAddModule(struct BusModuleNode *last,
		   const char *name,
		   int id,
		   int fd)
{
  struct BusModuleNode *newModule;
  
  newModule = (struct BusModuleNode *)
    malloc( sizeof( struct BusModuleNode ) );
  
  if (newModule == NULL)
    return NULL;
  else
    {
      newModule->moduleName = (char *)malloc(strlen(name)+1);
      if (newModule->moduleName == NULL)
	{
	  free( newModule );
	  return NULL;
	}
      strcpy(newModule->moduleName,name);
      
      if (last)
	newModule->next = last->next;
      else
	newModule->next = newModule;
      
      if (last)
	last->next = newModule;
      
     
      newModule->moduleId = id;
      newModule->fd = fd;
      
      newModule->Types = NULL;
      newModule->Messages = NULL;
      
      return newModule;
    }
}

struct BusModuleNode *
BusMasterFindModuleByName(struct BusModuleNode *last,
			  const char *name)
{
  struct BusModuleNode *current;
  
  if (last)
    current = last->next;
  else
    return NULL;
  
  while( current && current!=last && strcmp(current->moduleName,name))
    current = current->next;
  
  if (strcmp(current->moduleName,name))
    return NULL;
  else
    return current;
}

struct BusModuleNode *
BusMasterFindModuleById  (struct BusModuleNode *last,
			  int id)
{
  struct BusModuleNode *current;
  
  if (last)
    current = last->next;
  else
    return NULL;
  
  while( current && current!=last && (current->moduleId != id))
    current = current->next;
  
  if (current->moduleId != id)
    return NULL;
  else
    return current;
}

struct BusModuleNode *
BusMasterFindModuleByFd(struct BusModuleNode *last,
			int fd)
{
  struct BusModuleNode *cur;
  
  if(last == NULL)
    return NULL;
  else
    cur=last->next;
  
  while( cur && (cur != last) && (fd != cur -> fd) )
    cur = cur->next;
  
  if (fd == cur->fd)
    return cur;
  else
    return NULL;
}

int BusMasterAddRegisteredType(struct BusModuleNode *bmn,
				int typeId)
{
  struct BusRegisteredTypeNode *newNode;
  
  newNode = (struct BusRegisteredTypeNode *)
    malloc( sizeof(struct BusRegisteredTypeNode) );
  
  if (newNode != NULL)
    {
      newNode->next = bmn->Types;
      newNode->typeId = typeId;
      
      bmn->Types = newNode;
      
      return SBUSERROR_NOT;
    }
  else
    return SBUSERROR_NOMEMORY;
}

int BusMasterCheckRegisteredTypes(struct BusModuleNode *bmn,
				  int typeId )
/* is a given type understood by a module? */
/* returns 1 if the type is understood, 0 if the type is not */
{
  struct BusRegisteredTypeNode *current;
  
  if (bmn == NULL)
    return 0;
  if (bmn->Types == NULL)
    return 0;
  
  /*printf("Checking to see if module %s understands type %d.\n",
	 bmn->moduleName, typeId);*/
  
  if (bmn->Types->typeId == typeId)
    return 1;
  
  /*printf("Not type %d\n",bmn->Types->typeId);*/
  
  current = bmn->Types;
  
  do {
    current = current->next;
  } while( current && (current != bmn->Types) &&
	  (current->typeId != typeId) );
  
  /* we have either found the type, or reached the end of the understood
     types list */
  
  if(current && (current->typeId == typeId))
      return 1;       
  else
      return 0;
}

