#ifndef ERROR_H
#define ERROR_H

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
 * PURPOSE: Error.h - Declares routines for setting the global error handler.
 * NOTES:   Required source-code control string:
 *          "@(#)Error.h	2.1 /env/proj/archive/edss/src/pave/pave_drawmap/SCCS/s.Error.h 12/15/97 16:23:34"
 *          Usage:
 *            #include <Error.h>
 *            static void yourErrorHandler( int errorNumber,
 *                                          const char* message )
 *            {
 *              ...
 *            }
 *            Handler oldHandler = getErrorHandler();
 *            setErrorHandler( yourErrorHandler );
 *            int* p = NEW( int, 100000000 );
 *            setErrorHandler( oldHandler );
 * HISTORY: 03/93, Todd Plessel, EPA/MMTSI, Created.
 *          11/94, Todd Plessel, EPA/MMTSI, Modified for use as an example.
 *****************************************************************************/

/*================================= TYPES ===================================*/

typedef void (*ErrorHandler)( int errorNumber, const char* message );

/*=============================== FUNCTIONS =================================*/

extern void error( const char* message, ... );
extern ErrorHandler getErrorHandler( void );
extern void setErrorHandler( ErrorHandler anErrorHandler );
extern unsigned errors( void );

#ifdef __cplusplus
}
#endif

#endif /* ERROR_H */
