/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)map.c	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_graph/SCCS/s.map.c
 * Last updated: 12/15/97 16:24:49
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)map.c	2.1 /env/proj/archive/edss/src/pave/pave_graph/SCCS/s.map.c 12/15/97 16:24:49";

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

/*
MODIFICATION HISTORY:

WHO  WHEN       WHAT
---  ----       ----
SRT  04/06/95   Added #include "vis_proto.h"
*/

#include "vis_proto.h"

#define MAP_CMD "$BLT_WISH -f $EDSS_BINDIR/map.blt"

#define PAVE_SUCCESS 1
#define FAILURE 0

int map(float *x, float *y, int nlines, int *npoints,
char *title, char *xaxis_label, char *yaxis_label, char *message)
{
char *tmpfilename = NULL;
char *map_cmd = NULL;
FILE *tcl_input;
int status;
register int i, j, k;
pid_t pid;

status = FAILURE;

if ((tmpfilename = (char *) malloc(sizeof(char) * 512)) == NULL)
	goto ALLOCATE_MAP_ERROR;
if ((map_cmd = (char *) malloc(sizeof(char) * 512)) == NULL)
	goto ALLOCATE_MAP_ERROR;

pid = getpid();
sprintf(tmpfilename, "/tmp/map.blt.%d", pid);

sprintf(map_cmd, "%s < %s&", MAP_CMD, tmpfilename);

if ((tcl_input = fopen(tmpfilename, "w")) != NULL)
	{
	if (fprintf(tcl_input, "%s\n", tmpfilename) < 0)
		goto WRITE_MAP_ERROR;
	if (xaxis_label == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.xTitle \"%s\"\n",
			 "X Axis Label") < 0)
			goto WRITE_MAP_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.xTitle \"%s\"\n",
			 xaxis_label) < 0)
			goto WRITE_MAP_ERROR;
		}
	if (yaxis_label == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.yTitle \"%s\"\n",
			 "Y Axis Label") < 0)
			goto WRITE_MAP_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.yTitle \"%s\"\n",
			 yaxis_label) < 0)
			goto WRITE_MAP_ERROR;
		}
	if (title == NULL)
		{
		if (fprintf(tcl_input, "option add *graph.title \"%s\"\n",
			 " ") < 0)
			goto WRITE_MAP_ERROR;
		}
	else
		{
		if (fprintf(tcl_input, "option add *graph.title \"%s\"\n",
			 title) < 0)
			goto WRITE_MAP_ERROR;
		}

	k = 0;
	for (i = 0; i < nlines; i++)
		{
		if (fprintf(tcl_input, "set X%d { ", i+1) < 0)
			goto WRITE_MAP_ERROR;
		for (j = 0; j < npoints[i]; j++)
			{
			if (fprintf(tcl_input, "%g ", x[k+j]) < 0)
				goto WRITE_MAP_ERROR;
			}
		if (fprintf(tcl_input, "}\n") < 0)
			goto WRITE_MAP_ERROR;

		if (fprintf(tcl_input, "set Y%d { ", i+1) < 0)
			goto WRITE_MAP_ERROR;
		for (j = 0; j < npoints[i]; j++)
			{
			if (fprintf(tcl_input, "%g ", y[k+j]) < 0)
				goto WRITE_MAP_ERROR;
			}
		if (fprintf(tcl_input, "}\n") < 0)
			goto WRITE_MAP_ERROR;
				
		if (fprintf(tcl_input,
"$graph element create \"line%d\" -xdata $X%d -ydata $Y%d -symbol line -bg blue -foreground blue -linewidth 1\n",
		 i+1, i+1, i+1) < 0)
			goto WRITE_MAP_ERROR;
		k += npoints[i];
		}
	}
else
	goto CREATE_MAP_ERROR;

fflush(tcl_input); 
if (fclose(tcl_input) != 0)
	goto CLOSE_MAP_ERROR;

if (system(map_cmd) == 0)
	status = PAVE_SUCCESS;
if (tmpfilename != NULL)
	free(tmpfilename);
if (map_cmd != NULL)
	free(map_cmd);
return(status);

ALLOCATE_MAP_ERROR:
sprintf(message, "Error allocating file name.\n");
if (tmpfilename != NULL)
	free(tmpfilename);
if (map_cmd != NULL)
	free(map_cmd);
return(status);

CREATE_MAP_ERROR:
sprintf(message, "Error creating file: %s\n", tmpfilename);
if (tmpfilename != NULL)
	free(tmpfilename);
if (map_cmd != NULL)
	free(map_cmd);
return(status);

WRITE_MAP_ERROR:
sprintf(message, "Error writing to file: %s\n", tmpfilename);
fclose(tcl_input);
if (tmpfilename != NULL)
	free(tmpfilename);
if (map_cmd != NULL)
	free(map_cmd);
return(status);

CLOSE_MAP_ERROR:
sprintf(message, "Error closing file: %s\n", tmpfilename);
if (tmpfilename != NULL)
	free(tmpfilename);
if (map_cmd != NULL)
	free(map_cmd);
return(status);
}

