/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: %W%
 *     Pathname: %P%
 * Last updated: %G% %U%
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "%W% %P% %G% %U%";

		// use "pre-release" or "development" in the version names
		// for an unofficial, unreleased version
//#define PAVE_SHORT_VERSION "PAVE Development Version"            
//#define PAVE_LONG_VERSION  "PAVE Development Version %G% %U%" 
#define PAVE_SHORT_VERSION "PAVE v2.3.2"            
//#define PAVE_LONG_VERSION  PAVE_SHORT_VERSION" %G% %U%"
#define PAVE_LONG_VERSION  PAVE_SHORT_VERSION" December 8, 2004"
#define DATE_OF_COMPILATION "%G"


///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// File:	Main.cc 
// Author:	K. Eng Pua
// Date:	Dec 19, 1994
///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

#include "Main.h"
#ifdef linux
#include </usr/include/netdb.h>
#endif

typedef struct XINPUT_DATA
{
FILE *stream;	/* identifies the stream */
} XINPUT_DATA;



static DriverWnd *window;
static char logFile[256];
static time_t startSeconds;


static char *get_time_string(void)
{
static char time_string[128];
struct timeval tp;
struct timezone tzp;
struct tm *local;

gettimeofday(&tp, &tzp);
local = localtime(
#ifdef __osf__
(const int *)
#endif /* __osf__ */
(const time_t *)&tp.tv_sec);
strcpy(time_string, asctime(local));
if (time_string[strlen(time_string)-1] == '\n')
    time_string[strlen(time_string)-1] = '\0';
return time_string;
}


static char *get_user_at_host(void)
{
static char 	user_string[128];
static int 	first_time = 1;
struct passwd 	*pwd;
struct utsname 	uts;

if (first_time)
	{
   	pwd = getpwuid( getuid() );
   	uname(&uts);
   	sprintf(user_string, "%s@%s", pwd->pw_name, uts.nodename);
	first_time = 0;
	}

return user_string;
}


char *get_user_name(void)
{
static struct passwd *pwd = getpwuid(getuid());
return pwd->pw_name;
}


char *get_host(void)
{
static char 	user_string[128];
static int 	first_time = 1;

if (first_time)
	{
	struct utsname 	uts;
	uname(&uts);
   	sprintf(user_string, "%s", uts.nodename);
	first_time = 0;
	}

return user_string;
}


void pave_log_start(char *exe_name)
{
FILE *fp;
char dummy_host_name[256], dummy_path_name[256], dummy_file_name[256];
struct timeval tp;
struct timezone tzp;

parseLongDataSetName(exe_name,dummy_host_name,dummy_path_name,dummy_file_name);
logFile[0] = '\0';
if (dummy_path_name[0]) sprintf(logFile, "%s/", dummy_path_name);
strcat (logFile, ".");
strcat (logFile, dummy_file_name);
strcat (logFile, ".pave_usage.log");
fp = fopen(logFile, "a+");
if (fp == NULL) {
  fprintf(stderr,"Error writing to file %s\n",logFile);
  exit(-1);
}
fprintf(fp, 
        "Enter %s %s %s\n",
        PAVE_SHORT_VERSION,
        get_time_string(),
        get_user_at_host()
        );
gettimeofday(&tp, &tzp);
startSeconds = tp.tv_sec; 
fclose(fp);
chmod(logFile, (mode_t) S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
}


void pave_log_stop(void)
{
struct timeval tp;
struct timezone tzp;
FILE   *fp = fopen(logFile, "a+");
long   esec, hr, min, sec;

gettimeofday(&tp, &tzp);
esec = tp.tv_sec - startSeconds;
sec = esec%60;
min = ((esec-sec)/60)%60;
hr = (esec-min*60-sec)/3600;
fprintf(fp, 
        "Exit  %s %s %s %02ld:%02ld:%02ld clock time\n",
        PAVE_SHORT_VERSION,
        get_time_string(),
        get_user_at_host(),
	hr,
	min,
	sec
        );
fclose(fp);
}


void pave_version(void)
{
printf("%s\n", PAVE_LONG_VERSION);
fflush(stdout);
}


	// PAVE shouldn't run any more DAYS_TO_EXPIRATION or more 
        // days after Main.cc has been checked in, and PAVE has
	// been recompiled
int license_expiration_check(void) // returns number of days until expiration,
				   // if 0 license has expired
{
int 	compile_day = -1, 
	compile_month = -1, 
	compile_year = -1,
	compile_day_of_year = -1,
	current_day_of_year, 
	current_year,
	expiration_year,
	expiration_day_of_year,
	days_since_010195,
	days_from_010195_until_expiration;
char	local_date_of_comp[20],
	*cp;
struct 	timeval tp;
struct 	timezone tzp;
struct 	tm *local;

gettimeofday(&tp, &tzp);
local = localtime(
#ifdef __osf__
(const int *)
#endif /* __osf__ */
(const time_t *)&tp.tv_sec);
current_year = local->tm_year + ((local->tm_year>50) ? 1900 : 2000);
current_day_of_year = local->tm_yday+1;

	// DATE_OF_COMPILATION is determined by SCCS above;
	// it is of the form "2/21/96" on some machines (including DEC and HP)
	// and of the form "21 Feb 1996" on others (including SGI and SUN)
	// and of course it is of the form "22 Apr 1996" if the code is checked out
#define DAYS_TO_EXPIRATION 120
if (!strcmp("\%G\%", DATE_OF_COMPILATION))
	{
#ifdef DIAGNOSTICS
	fprintf(stderr, "Not checked in - allowing license!\n"); 
#endif // DIAGNOSTICS
	return 1;
	}
strcpy(local_date_of_comp, DATE_OF_COMPILATION);

if (cp = strrchr(local_date_of_comp, '/')) // "2/21/96" case
	{
	compile_year = 1900 + atoi(cp+1);
	if (current_year < 1996) // Year 2000 date check
		compile_year+=100;
	*cp = '\0';
	cp = strrchr(local_date_of_comp, '/');
	compile_day = atoi(cp+1);
	*cp = '\0';
	compile_month = atoi(local_date_of_comp);
	}
else					   // "21 Feb 1996" case
	{
	cp = strrchr(local_date_of_comp, ' ');
	compile_year = atoi(cp+1);
	*cp = '\0';
	cp = strrchr(local_date_of_comp, ' ');
	if (!strcasecmp(cp+1, "Jan")) compile_month = 1; else
	if (!strcasecmp(cp+1, "Feb")) compile_month = 2; else
	if (!strcasecmp(cp+1, "Mar")) compile_month = 3; else
	if (!strcasecmp(cp+1, "Apr")) compile_month = 4; else
	if (!strcasecmp(cp+1, "May")) compile_month = 5; else
	if (!strcasecmp(cp+1, "Jun")) compile_month = 6; else
	if (!strcasecmp(cp+1, "Jul")) compile_month = 7; else
	if (!strcasecmp(cp+1, "Aug")) compile_month = 8; else
	if (!strcasecmp(cp+1, "Sep")) compile_month = 9; else
	if (!strcasecmp(cp+1, "Oct")) compile_month = 10; else
	if (!strcasecmp(cp+1, "Nov")) compile_month = 11; else
	if (!strcasecmp(cp+1, "Dec")) compile_month = 12; 
	*cp = '\0';
	compile_day = atoi(local_date_of_comp);
	}

compile_day_of_year = 0;
if (compile_month > 1) compile_day_of_year+=31;
if (compile_month > 2) compile_day_of_year+=29;
if (compile_month > 3) compile_day_of_year+=31;
if (compile_month > 4) compile_day_of_year+=30;
if (compile_month > 5) compile_day_of_year+=31;
if (compile_month > 6) compile_day_of_year+=30;
if (compile_month > 7) compile_day_of_year+=31;
if (compile_month > 8) compile_day_of_year+=31;
if (compile_month > 9) compile_day_of_year+=30;
if (compile_month > 10) compile_day_of_year+=31;
if (compile_month > 11) compile_day_of_year+=30;
compile_day_of_year+=compile_day;

expiration_day_of_year = (compile_day_of_year+DAYS_TO_EXPIRATION)%365;
expiration_year = compile_year+(compile_day_of_year+DAYS_TO_EXPIRATION)/365;

days_since_010195 = (current_year-1995)*365+current_day_of_year;
days_from_010195_until_expiration = 
		    (expiration_year-1995)*365+expiration_day_of_year;

#ifdef DIAGNOSTICS
fprintf(stderr, "         compile_month == %d\n", compile_month); 
fprintf(stderr, "           compile_day == %d\n", compile_day); 
fprintf(stderr, "          compile_year == %d\n", compile_year); 
fprintf(stderr, "   compile_day_of_year == %d\n\n", compile_day_of_year); 
fprintf(stderr, "          current_year == %d\n", current_year); 
fprintf(stderr, "   current_day_of_year == %d\n\n", current_day_of_year); 
fprintf(stderr, "expiration_day_of_year == %d\n", expiration_day_of_year); 
fprintf(stderr, "       expiration_year == %d\n", expiration_year); 
fprintf(stderr, " Days until expiration == %d\n",
			days_from_010195_until_expiration-days_since_010195);
#endif // DIAGNOSTICS

return days_from_010195_until_expiration-days_since_010195;
}


char *get_local_host_name(void)
{
static char     hostname[128];
struct utsname  uts;

uname(&uts);
sprintf(hostname, "%s", uts.nodename);
return hostname;
}

#ifndef __osf__
char *get_local_ip_number(void)
{
struct 		hostent *h_info;
struct 		in_addr *hptr;
static char 	local_ip_number[64];

h_info = gethostbyname(get_local_host_name());
hptr = (struct in_addr *) *h_info->h_addr_list;
sprintf(local_ip_number, "%s", inet_ntoa(*hptr));
return local_ip_number;
}
#endif // #ifndef __osf__


void init(void)
{
#ifndef __osf__
char 	*mcnc_subnets[] = { "198", "128" },
	*local_ip_number = get_local_ip_number(),
	versions[160];
int 	i, len, at_mcnc = 0;

strcpy(versions, PAVE_SHORT_VERSION);
strcat(versions, PAVE_LONG_VERSION);
len = strlen(versions);
for (i = 0; i < len; i++) versions[i] = toupper(versions[i]);

	// is this an unreleased version?
if (strstr(versions, "PRE") || strstr(versions, "DEVELOPMENT"))	
	{
		// are we at MCNC?
	for (i = 0; i < sizeof(mcnc_subnets)/sizeof(mcnc_subnets[0]); i++)
		if (!strncmp(local_ip_number, mcnc_subnets[i], strlen(mcnc_subnets[i])))
			at_mcnc = 1;
	if (!at_mcnc) exit(1);
	}
#endif // #ifndef __osf__
}


void pave_startup_msg()
{
printf("\n");
printf("    Package for Analysis and Visualization of Environmental data        \n");
printf("               %s\n",                                     PAVE_LONG_VERSION);
printf("                                                                        \n");
printf("                For further information on PAVE:                        \n");
printf("Copyright Info: type -copyright in PAVE's standard input                \n");
printf("         Usage: type -usage in PAVE's standard input                    \n");
printf("    User Guide: http://www.cep.unc.edu/empd/EDSS/pave_doc/index.shtml       \n");
printf("           FAQ: http://www.cep.unc.edu/empd/EDSS/pave_doc/Pave.FAQ.html   \n");
printf(" Bugs and Info: http://bugz.unc.edu, http://www.cmascenter.org     \n\n");
fflush(stdout);
}


void pave_copyright(void)
{
//printf("\n");
//printf("       Package for Analysis and Visualization of Environmental data        \n");
//printf("                  %s\n",                                     PAVE_LONG_VERSION);
//printf("               Copyright 1996 MCNC, Hill under terms of the GNU Public License.  See gpl.txt for more details.                    \n");
//printf("                                                                           \n");
//printf("           Additional Copyright Information:                               \n");
//printf("                            BLT AT&T Bell Laboratories                     \n");
//printf("                         imconv San Diego Supercomputer Center             \n");
//printf("           MapUtilities library Lockheed/Martin Technical Services         \n");
//printf("                         netCDF UCAR/Unidata                               \n");
//printf("                         PLplot Digital Equipment Corporation              \n");
//printf("    pnmtops, pnmdepth, xwdtopnm Jef Poskanzer                              \n");
//printf("                   PROJ library United States Geological Survey            \n");
//printf("Several of Doug Young's Widgets Prentice Hall                              \n");
//printf("Tcl, Tk, mpeg_encode, mpeg_play The Regents of the University of California\n");
//printf("                                                                           \n");
//printf("                   For further information on PAVE:                        \n");
//printf("       User Guide: http://www.emc.mcnc.org/EDSS/pave_doc/Pave.html       \n");
//printf("              FAQ: http://www.emc.ncsc.org/EDSS/pave_doc/Pave.FAQ.html   \n");
//printf("            Email: pave@emc.mcnc.org                                   \n");
//fflush(stdout);
}

	// SRT added 950913 to catch any input from stdin
void read_input(caddr_t client_data, int *, XtInputId *)
{
static char line[2048];
XINPUT_DATA *p_xp = (XINPUT_DATA *) client_data;
int len;

	// read the input line
if (fgets(line, sizeof(line), p_xp->stream) == NULL)
	{
	// perror ("fgets");
	return; // exit(1);
	}

	// strip off the '\n' at the end
len = strlen(line);
if (line[len-1] == '\n') line[len-1]='\0';

window->parseMessage(line);
}


#if (XlibSpecificationRelease>=5)
int main ( int argc, char **argv )
#else
int main ( unsigned int argc, char *argv[] )
#endif
{

   XtAppContext this_app; // SRT added 950913
   XEvent theEvent; 	  // SRT added 950913
   int i;

   XINPUT_DATA xpinfo;    // SRT added 950913

   char errorMsg[512];

   errorMsg[0] = '\0';

   //verify_copyright_file();
   init();
   pave_startup_msg();

#ifdef SQUAT	// removed for public domain version 961024 SRT
   int days_until_expiration = license_expiration_check();
   if (days_until_expiration <= 0)
	{
	fprintf(stderr, 
               "\nWARNING:  Your license to run this software has expired.\n");
	fprintf(stderr, "Please contact MCNC's Carolina Environmental Program\n");
	fprintf(stderr, "at pave@emc.mcnc.org or (919) 248-1100 to renew.\n");
	exit(1);
	}
   if (days_until_expiration <= 31)
	{
	fprintf(stderr, 
        "\nWARNING:  Your license to run this software will expire\nin %d day%s.",
		days_until_expiration, (days_until_expiration>1)?"s":"");
	fprintf(stderr, "  Please contact MCNC's Environmental \n");
	fprintf(stderr, "Programs Group at pave@emc.mcnc.org or (919) 248-1100 \nto renew.\n\n");
	}
#endif // #ifdef SQUAT

for (i = 0; i < argc; i++)
        if 	(    
		(!strcasecmp(argv[i], "-help")) ||
                (!strcasecmp(argv[i], "-fullhelp")) ||
                (!strcasecmp(argv[i], "-usage")) ||
		(!strcasecmp(argv[i], "help")) ||
                (!strcasecmp(argv[i], "fullhelp")) ||
                (!strcasecmp(argv[i], "usage"))
		)
                {
                pave_usage();
		exit(1);
                }

//   pave_log_start(argv[0]); // disabled by A.Traynov 04/10/2002

#ifdef DIAGNOSTICS
   printf("Enter '");
   for (i = 0; i < argc; i++)
     {
     if (i) printf(" ");
     printf("%s", argv[i]);
     }
   printf("'\n");
   fflush(stdout);
#endif // DIAGNOSTICS

   AppInit *uam = new AppInit ( "EDSS_Pave" );
   uam->initialize(&argc, argv);

   char title[100];
   sprintf(title, "%s (%s)", PAVE_SHORT_VERSION, get_user_at_host());

   char historyfile[255];
   sprintf(historyfile, "%s/.pave_history_rc", getenv("HOME")); 

#ifdef DIAGNOSTICS
   fprintf(stderr, "using historyfile '%s'\n", historyfile);
#endif

   window  = new DriverWnd ( uam, title, historyfile, argc, argv, errorMsg);
   if (errorMsg[0])
	{
	fprintf(stderr, "%s\n", errorMsg);
	delete uam;
	delete window;
	return 1;
	}

		// to catch standard input add this callback
   xpinfo.stream = stdin; 
   XtAppAddInput(
		(XtAppContext) uam->appContext(),	// app_context
		(int) fileno(stdin), 			// source
		(XtPointer) XtInputReadMask, 		// condition
		(XtInputCallbackProc) read_input, 	// proc
		(XtPointer) &xpinfo			// closure
		);

   uam->realizeWidget();

   // uam->handleEvents(); SRT 950913 removed and replaced with code below 
   this_app = uam->appContext();		// added SRT 950913
   while (1)					// added SRT 950913
	{					// added SRT 950913
	XtAppNextEvent (this_app, &theEvent);	// added SRT 950913
	XtDispatchEvent (&theEvent);		// added SRT 950913
	}					// added SRT 950913
}

