/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)BusConnect.cc	2.1
 *     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.BusConnect.cc
 * Last updated: 12/15/97 16:31:55
 *
 * Made available by MCNC and the Carolina Environmental Program of UNC Chapel
 * Hill under terms of the GNU Public License.  See gpl.txt for more details.
 *
 * See file COPYRIGHT for license information on this and supporting software.
 *
 * Carolina Environmental Program
 * University of North Carolina at Chapel Hill
 * 137 E. Franklin St.
 * Chapel Hill, NC 27599-6116
 *
 * See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)BusConnect.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.BusConnect.cc 12/15/97 16:31:55";

////////////////////////////////////////////////////////////
// BusConnect.C: 
//
////////////////////////////////////////////////////////////
//
// Revision History
// SRT  950523  Added check of REMOTEACCESS environment variable
//		so bus warning msgs are only printed out if the
//		PAVE wrapper script thought we were supposed to
//		have remote access
//
/////////////////////////////////////////////////////////////

#include "BusConnect.h"

static BusConnect *theBus;
static struct BusData bd;

extern void init_Browser(struct BusData *bd); // added 960411 SRT

void BusConnect::busCallback(char *) { } // SRT 950929


/***** Bus Callbacks *****/
void pave_callback( struct BusData *bd, struct BusMessage *bmsg )
/* Call-back function for a ASCII data type called pave
   (that is, a simple ascii string, in plain English)
   This function a) checks the caller-id,
                 b) establishes the requested distribution mode,
                 c) informs user that message was received,
                 d) prints the message and flushes output buffer. */
{
  char *from, *to;

  // Identify message sender 
  from = BusFindModuleById( bd, bmsg->fromModule );
  if (from == NULL)
      from = "Unidentified source (source no longer connected to bus?)";

  // Identify message distribution mode 
  if (bmsg->toModule == BusBroadcast)
    to   = "Everybody";
  else if (bmsg->toModule == BusByType)
    to   = "modules understanding type";
  else if (bmsg->toModule == BusBounce)
    to   = "myself (Bounce)";
  else if (bmsg->toModule == bd->moduleId)
    to   = "myself (my module id)";
  else if (bmsg->toModule != bd->moduleId)
    to   = BusFindModuleById( bd, bmsg->  toModule );
  else
    to = "Unknown (not good...)";

#ifdef DIAGNOSTICS
  // Inform user that msg was received, from whom it was received, and
  //   to whom it was directed (distribution mode) 
  printf("PAVE message received:\n");
  printf("\tFrom: %s\n", from );

  if (to != NULL)
    printf("\tTo  : %s\n", to );

  // Print message 
  printf("\"%s\"\n", bmsg->message);

  // Flush print buffer to ensure that msg is printed promptly 
  fflush(stdout);
#endif

  theBus->busCallback(bmsg->message); 

}


void testcallback()
{
  theBus->busCallback("TestCallback");
}



BusConnect::BusConnect ( AppInit *app ) 
{
   bd.name = strdup("pave");

   // attempt to attach to the bus
   int err = BusXtInitialize( &bd, app->appContext());
   if (err != SBUSERROR_NOT) {
           fprintf(stderr, "Did not connect to the software bus.\n");
	   isConnected_ = 0;
   }
   else {

	   isConnected_ = 1;

#ifdef DIAGNOSTICS
	printf("Bus Connection established.\n");

	// Get the handle for standard bus-managed pave data type
	printf("Find id for type pave\n");
#endif

	int typeId = BusFindTypeByName( &bd, "PAVE_TYPE" );

#ifdef DIAGNOSTICS
	printf("Setting callback for type pave\n");
#endif

	BusAddTypeCallback( &bd, typeId, pave_callback );

#ifdef DIAGNOSTICS
	printf("Callback for pave established.\n");
#endif


	init_Browser(&bd); // added 950921 SRT
   }

   theBus = this;
   bd_ = &bd;
}


