/////////////////////////////////////////////////////////////
//
// Project Title: Environmental Decision Support System
//         File: @(#)LinkedList.cc	2.1
//     Pathname: /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.LinkedList.cc
// Last updated: 12/15/97 16:33:12
//
// Made available by MCNC and the Carolina Environmental Program of UNC Chapel
// Hill under terms of the GNU Public License.  See gpl.txt for more details.
//
// See file COPYRIGHT for license information on this and supporting software.
//
// Carolina Environmental Program
// University of North Carolina at Chapel Hill
// 137 E. Franklin St.
// Chapel Hill, NC 27599-6116
//
// See http://www.cep.unc.edu, http://www.cmascenter.org, http://bugz.unc.edu
//
/////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// --------------------
// Modification History
// --------------------
//
// ---  ----    ----
// Who  When    What
// ---  ----    ----
// SRT  950607  Added removeCurrent() routine
//
//////////////////////////////////////////////////////////////////////////////


static const char *SCCS_ID = "@(#)LinkedList.cc	2.1 /env/proj/archive/edss/src/pave/pave_motif/SCCS/s.LinkedList.cc 12/15/97 16:33:12";

/////////////////////////////////////////////////////////////
//
// LinkedList.C
// Steve Thorpe
// thorpe@ncsc.org
// NCSC Environmental Programs, MCNC
// May 23, 1995
//
/////////////////////////////////////////////////////////////
//
// LinkedList Class 
//
// Portions from Roger Sessions' "Class Construction in C
// and C++", copyright 1992 by Prentice-Hall
//
/////////////////////////////////////////////////////////////
//
// Revision History
// SRT	950523	Implemented
//
/////////////////////////////////////////////////////////////


#include "LinkedList.h"


linkedList::linkedList()
{
currentLink = headLink = tailLink = 0;
nlinks = 0;
max = MAX_INT;
}


linkedList::~linkedList()
{
// SRT replace with freeContents()    while(removeHead());
freeContents();
}


void linkedList::freeContents()
{
baseType *thisItem;
while(thisItem = removeHead()) delete thisItem;
}


baseType *linkedList::head()
{
if (!nlinks) return 0;
currentLink = headLink;
return ((linkp *)currentLink)->contents;
}


baseType *linkedList::tail()
{
if (!nlinks) return 0;
currentLink = tailLink;
return ((linkp *)currentLink)->contents;
}

int linkedList::length()
{
return nlinks;
}


void linkedList::setMax(int newMax)
{
max = newMax;
}


int linkedList::left()
{
return max - length();
}


baseType *linkedList::next()
{
if (!nlinks) return 0;
if (((linkp *)currentLink)->next)
	{
	currentLink = ((linkp *)currentLink)->next;
	return ((linkp *)currentLink)->contents;
	}
else
	return 0;
}


baseType *linkedList::previous()
{
if (!nlinks) return 0;
if (((linkp *)currentLink)->previous)
	{
	currentLink = ((linkp *)currentLink)->previous;
	return ((linkp *)currentLink)->contents;
	}
else
	return 0;
}


baseType *linkedList::retrieve()
{
if (!nlinks) return 0;
return ((linkp *)currentLink)->contents;
}


baseType *linkedList::replace(baseType *newElement)
{
if (!nlinks) return 0;
((linkp *)currentLink)->contents = newElement;
return ((linkp *)currentLink)->contents;
}


baseType *linkedList::promoteTail()
{
linkp *oldTail;

if (!nlinks) return 0;
if (nlinks == 1) return ((linkp *)headLink)->contents;
oldTail = (linkp *) tailLink;
tailLink = ((linkp *)tailLink)->previous;
((linkp *)oldTail->previous)->next = 0;
oldTail->previous  = 0;
oldTail->next = headLink;
((linkp *)headLink)->previous = oldTail;
headLink = oldTail;
currentLink = headLink;
return ((linkp *)currentLink)->contents; // added SRT
}


baseType *linkedList::addHead(baseType *newElement)
{
linkp *newLink = new linkp(newElement);
assert(newLink);

if (!left())
	{
	head();
	return(replace(newElement));
	}

if (head())
	((linkp *)currentLink)->previous = newLink;
else
	tailLink = newLink;

newLink->next = currentLink;
headLink = currentLink = newLink;
nlinks++;

return ((linkp *)currentLink)->contents;
}


baseType *linkedList::addTail(baseType *newElement)
{
linkp *newLink = new linkp(newElement);
assert(newLink);

if (!left())
	{
	tail();
	return(replace(newElement));
	}

if (tail())
	((linkp *)currentLink)->next = newLink;
else
	headLink = newLink;

newLink->previous = currentLink;
tailLink = currentLink = newLink;
nlinks++;

return ((linkp *)currentLink)->contents;
}


baseType *linkedList::findAndRemoveLink(void *findme) // SRT DOES THIS WORK?
{
baseType *nextElement;
head();
for (;;)
	{
	nextElement = retrieve();
	if (!nextElement) return (baseType *) 0;
	if (nextElement->match(findme)) 
		{
		if (nlinks == 1)
			{
			delete headLink;
			headLink = tailLink = currentLink = 0;
			}
		if (nlinks > 1)
			{
			if (((linkp *)currentLink)->previous) 
			  ((linkp *)((linkp *)currentLink)->previous)->next = 
					((linkp *)currentLink)->next;
			else
				headLink = ((linkp *)currentLink)->next;
		
			if (((linkp *)currentLink)->next) 
			  ((linkp *)((linkp *)currentLink)->next)->previous = 
					((linkp *)currentLink)->previous;
			else
				tailLink = ((linkp *)currentLink)->previous;
			delete ((linkp *)currentLink);
			head();
		}
		nlinks--;

#ifdef DIAGNOSTICS
		fprintf(stderr, "in findAndRemoveLink:\n");
		nextElement->print(stderr);
#endif // DIAGNOSTICS

		return nextElement; // match
		}
	if (isTail()) return (baseType *) 0;                // no match
	next();
	}
}





baseType *linkedList::removeHead()
{
baseType *thisItem;

if (!nlinks) return 0;
thisItem = head();
if (nlinks == 1)
	{
	delete headLink;
	headLink = tailLink = currentLink = 0;
	}
if (nlinks > 1)
	{
	next();
	delete headLink;
	headLink = currentLink;
	((linkp *)headLink)->previous = 0;
	}
nlinks--;
return thisItem;
}


int linkedList::isTail()
{
return (currentLink == tailLink);
}


void linkedList::print(FILE *output)
{
baseType *thisItem;

thisItem = head();
if (thisItem)
	{
	while (thisItem)
		{
		thisItem->print(output);
		thisItem = next();
		}
	}
}



baseType *linkedList::find(void *findme)
{
baseType *nextElement;
head();
for (;;)
	{
	nextElement = retrieve();
	if (!nextElement) return (baseType *) 0;
	if (nextElement->match(findme)) return nextElement; // match
	if (isTail()) return (baseType *) 0;                // no match
	next();
	}
}


        // override baseType's method
char *linkedList::getClassName()
{
static char *myName = "linkedList";
return myName;
}
