C##############################################################################
*DK CKSUMER
      SUBROUTINE CKSUMER(MODE,CKSUM,SCKSUM)
C*********************************************************************
C**   DESCRIPTION: Calculates a checksum value for any of a discrete
C**   set of files. Calculation result should be used either to create
C**   a checksum value to be written in the file, or to verify a value
C**   which has been read in.
C**
C**     1 Revision History:
C**       No. Date     Who  What
C**       --  -------- ---  ----------------------------------------
C**       04  90130    MCB  MODIFIED TO WORK FOR ONLY 2 MET FILES
C**                         FOR AQM STRUCTURE
C**                         1. REMOVED MODE CALLS FOR 5 AND 6 (THE
C**                            MET NOW COMES IN TWO FILES ONLY)
C**                         2. CHANGED CHECKSUM LOGIC FOR THE EVEN
C**                            AND ODD MET FILE CALCULATIONS
C**        3  07/10/89 JNM  removed error traps for indefinites.
C**        2  07/03/89 JNM  added mode switches 7,8
C**        1  04/27/89 JKV  created module as part of RADM1 testing.
C**
C**      2 Input: first argument is mode switch indicating which file's
C**        data space is to be check-summed.  Mode codes are as follow.
C**           1  -  first initial conditions file
C**           2  -  second initial conditions file
C**           3  -  first met file, #1 even hours
C**           4  -  second met file, #2 odd hours
C**           7  -  source file, hour a
C**           8  -  source file, hour b
C**
C**      3 Output: second argument is checksum as calculated.
C**                third argument is string of length 12 containing
C**                the encoded ASCII value of the checksum calculated.
C**                AS E12.5:  +1.23456E+01
C**
C**      4 Called by:
C**
C**      5 Calls:
C**
C**********************************************************************
C**   INCLUDE FILES:
      include "a_params.com"
      include "a_cmmns.com"
c**********************************************************************
      INTEGER MODE
      REAL CKSUM
      CHARACTER*12  SCKSUM
      LOGICAL     BLNKAL
C
C BRANCH TO FILE SPECIFIC CALCULATIONS BASED ON VALUE OF MODE.
      GOTO (1,2,3,4,5,6)   MODE
C
C MODE OF ZERO ALLOWS ACCESS TO ENCODE OR DECODE UTILITY FUNCTION.
C WITHIN THIS UTILITY FUNCTION, CONTROL IS BASED ON SCKSUM BEING BLANK.
      IF(BLNKAL(SCKSUM,12)) THEN
          WRITE(SCKSUM,100) CKSUM
      ELSE
          READ(SCKSUM,100) CKSUM
      ENDIF
      RETURN
C
C   COMPUTE CHECKSUM FOR FIRST IC OR CSV FILE
1     CKSUM = 0.0
      DO 108, I = 1, 3
        DO 108, L = LC3D1S, LC3D1S+2
          DO 108, K = 1, 3
            DO 108, J = 1, 3
              CKSUM = CKSUM + VCI(J,K,L,I)
  108 CONTINUE
      DO 109, I = IMAX-2, IMAX
        DO 109, L = LC3D1E-2, LC3D1E
          DO 109, K = KMAX-2,KMAX
            DO 109, J = JMAX-2, JMAX
              CKSUM = CKSUM + VCI(J,K,L,I)
  109 CONTINUE
      WRITE(SCKSUM,100) CKSUM
110   RETURN
C
C   COMPUTE CHECKSUM FOR SECOND IC OR CSV FILE
2     CKSUM = 0.0
      DO 208, I = 1, 3
        DO 208, L = LC3D2S, LC3D2S+2
          DO 208, K = 1, 3
            DO 208, J = 1, 3
              CKSUM = CKSUM + VCI(J,K,L,I)
  208 CONTINUE
      DO 209, I = IMAX-2, IMAX
        DO 209, L = LC3D2E-2, LC3D2E
          DO 209, K = KMAX-2,KMAX
            DO 209, J = JMAX-2, JMAX
              CKSUM = CKSUM + VCI(J,K,L,I)
  209 CONTINUE
      WRITE(SCKSUM,100) CKSUM
210   RETURN
C
C   COMPUTE CHECKSUM FOR FIRST MET FILE, EVEN
3     CKSUM = 0.0
      DO 308, I = 1, 3
        DO 308, L = LM3D1S, LM3D1S+1            !90130 MCB
          DO 308, K = 1, 3
            DO 308, J = 1, 3
              CKSUM = CKSUM + VMI(J,K,L,I)
  308 CONTINUE
      DO 309, I = IMAX-2, IMAX
        DO 309, L = LM3D1E-2, LM3D1E
          DO 309, K = KMAX-2, KMAX
            DO 309, J = JMAX-2, JMAX
              CKSUM = CKSUM + VMI(J,K,L,I)
  309 CONTINUE
      WRITE(SCKSUM,100) CKSUM
310   RETURN
C
C   COMPUTE CHECKSUM FOR SECOND MET FILE, ODD
4     CKSUM = 0.0
      DO 408, I = 1, 3
        DO 408, L = LM3D2S, LM3D2S+1              ! 90130 MCB
          DO 408, K = 1, 3
            DO 408, J = 1, 3
              CKSUM = CKSUM + VMI(J,K,L,I)
  408 CONTINUE
      DO 409, I = IMAX-2, IMAX
        DO 409, L = LM3D2E-2, LM3D2E
          DO 409, K = KMAX-2, KMAX
            DO 409, J = JMAX-2, JMAX
              CKSUM = CKSUM + VMI(J,K,L,I)
  409 CONTINUE
      WRITE(SCKSUM,100) CKSUM
410   RETURN
C
C   COMPUTE CHECKSUM FOR HRA OF EMISSIONS FILE
5     CKSUM = 0.0
      DO 708 JB=1,3
       JD=JMAX+1-JB
       DO 708 KB=1,3
        KD=KSRCMAX+1-KB
        DO 708 LB=1,3
         LD=LEMIS+1-LB
         DO 708 IB=1,3
           ID=IMAX+1-IB
           CKSUM = CKSUM + SOURCE(JB,KB,LB,IB) +
     &             SOURCE(JD,KD,LD,ID)
708   CONTINUE
      WRITE(SCKSUM,100) CKSUM
710   RETURN
C   COMPUTE CHECKSUM FOR HRB OF EMISSIONS FILE
6     CKSUM = 0.0
      DO 808 JB=1,3
       JD=JMAX+1-JB
       DO 808 KB=1,3
        KD=KSRCMAX+1-KB
        DO 808 LB=1,3
         LD=LEMIS+1-LB
         DO 808 IB=1,3
           ID=IMAX+1-IB
           CKSUM = CKSUM + SOURCE(JB,KB,LB,IB) +
     &             SOURCE(JD,KD,LD,ID)
808   CONTINUE
      WRITE(SCKSUM,100) CKSUM
810   RETURN
100   FORMAT(E12.5E2)
      END
