/****************************************************************************
 *
 * Project Title: Environmental Decision Support System
 *
 *         File: @(#)asc_to_m3io.c	1.1
 *     Pathname: /tmp_mnt/200p_scratch/thorpe/ascii_to_m3io/SCCS/s.asc_to_m3io.c
 * Last updated: 17 May 1996 20:01:25
 *
 * COPYRIGHT (C) 1995, MCNC--North Carolina Supercomputing Center
 * All Rights Reserved
 *
 * See file COPYRIGHT for conditions of use.
 *
 * Environmental Programs Group
 * MCNC--North Carolina Supercomputing Center
 * P.O. Box 12889
 * Research Triangle Park, NC  27709-2889
 *
 * env_progs@mcnc.org
 *
 ****************************************************************************/

static const char *SCCS_ID = "@(#)asc_to_m3io.c	1.1 /tmp_mnt/200p_scratch/thorpe/ascii_to_m3io/SCCS/s.asc_to_m3io.c 17 May 1996 20:01:25";

/*
asc_to_m3io.c
Steve Thorpe
MCNC Environmental Programs
thorpe@ncsc.org
(919) 248-1161

This will convert Gopal's ROM data into IO/API netCDF formatted
data suitable for usage within PAVE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "iodecl3.h"

#define IDIM 128
#define JDIM 126


int usage(void)
{
fprintf(stderr, "Usage: asc_to_m3io <ascii file name> <netcdf file name> <nhours>\n");
fprintf(stderr, "NOTE: files are assumed IDIM 128 AND JDIM 126, stored i within j\n");
return 0;
}


int errmsg(char *s)
{
fprintf(stderr, "%s\n", s);
return 1;
}


int main(int argc, char *argv[])
{
IOAPI_Bdesc3 	ib3;
IOAPI_Cdesc3 	ic3;
int 		hour, i, j, nhours;
char 		ascfname[256], netfname[256], line[256], e[256], 
		specname[256], date[6], time[7], bfile[256];
float 		data[JDIM][IDIM];
FILE		*fp;
static char	logicalName[256];

if (argc != 4)
	return usage();

init3c();

	sprintf(bfile, "BFILE");

	memset ((void *) &ib3, (int)NULL, (size_t) sizeof(IOAPI_Bdesc3));
	memset ((void *) &ic3, (int)NULL, (size_t) sizeof(IOAPI_Cdesc3));

	sprintf(ascfname, argv[1]);
	sprintf(netfname, argv[2]);
	nhours = atoi(argv[3]);

	if (!(fp = fopen(ascfname, "r")))
		{
		sprintf(e, "Couldn't open input data file %s !", ascfname);
		return errmsg(e);
		}
	fprintf(stderr, "Converting %s...\n", ascfname);


	for (hour=0; hour<nhours; hour++)
		{
		if (!fgets(line, sizeof(line), fp))
		  {
		  sprintf(e, "Couldn't read date line from %s !", ascfname);
		  return errmsg(e);
		  }
		date[0] = line[22];
		date[1] = line[23];
		date[2] = line[24];
		date[3] = line[25];
		date[4] = line[26];
		date[5] = '\0';
		time[0] = line[28];
		time[1] = line[29];
		time[2] = line[30];
		time[3] = line[31];
		time[4] = line[32];
		time[5] = line[33];
		time[6] = '\0';

		if (!fgets(line, sizeof(line), fp))
		  {
		  sprintf(e, "Couldn't read species line from %s !", ascfname);
		  return errmsg(e);
		  }
		sscanf(&line[29], "%s", specname);
		fprintf(stderr, "date == '%s', time = '%s', species = '%s'\n", 
			date, time, specname);


		ib3.wdate = 1900000+atoi(date);
		ib3.wtime = atoi(time);
		if (hour == 0)
			{
			ib3.xorig = -99.0; /* as per Carlie Coats */
			ib3.yorig =  26.0; /* as per Carlie Coats */
			ib3.xcell =  -0.25; /* as per Carlie Coats */
			ib3.ycell =   0.1666667; /* as per Carlie Coats */
			ib3.ftype = GRDDED3;
			GETDTTIME(&ib3.cdate, &ib3.ctime);
			ib3.sdate = 1900000+atoi(date);
			ib3.stime = atoi(time);
			ib3.tstep = 10000;
			ib3.mxrec = 24;
			ib3.nvars = 1;
			ib3.ncols = IDIM;
			ib3.nrows = JDIM;
			ib3.nlays = 1;
			ib3.gdtyp = 1;
			ib3.vtype[0] = M3REAL;
			strcpy(ic3.vname[0], specname);
			strcpy(ic3.units[0], "PPM");
			sprintf(ic3.vdesc[0], "%s from asc_to_m3io", specname);

                	/*
                	logicalName is declared static as per the man pages
                	for putenv's mention of a potential error:

                	int putenv(char *string)

                	A potential error is to  call  putenv()  with  
			an  automatic variable  as  the  argument,  
			then exit the calling function while string is 
			still part of the environment.
                	*/
                	sprintf(logicalName, "%s=%s", bfile, netfname);
                	putenv(logicalName);

			if (!open3c(bfile, &ib3, &ic3, FSNEW3, "asc_to_m3io"))
		  	  {
		  	  sprintf(e, "Couldn't open %s !", netfname);
		  	  return errmsg(e);
		  	  }
			} 

		for (j = 0; j < JDIM; j++)
		  for (i = 0; i < IDIM; i++)
		    fscanf(fp, "%f", &data[j][i]);


		if (!write3c(bfile, "ALL", 
		             ib3.wdate, ib3.wtime, data))
		  	{
		  	sprintf(e, "Couldn't write to %s !", netfname);
		  	return errmsg(e);
		  	}

		fgets(line, sizeof(line), fp); /* to polish off that last \n */
		}

	fclose(fp);
	close3c(bfile);

shut3c();
}
