
      PROGRAM TRANCONC
C
C******
C**   DESCRIPTION:  Convert SAQM concentation file
C**     to netCDF format for models3.
C**
C**     1 Revision History:
C**       No. Date     Who     What
C**       --  ----     ---     ----
C**        0  2/17/92  HWANG   BEGIN PROGRAM
C**        1 10/25/96  CAREYJ  Convert SAQM output to Models-3 format,
C**				including hourly average data array
C**
C**
C**      2 Calls the following subroutines: BUFIN CKSUMER
C******
C**  CALL FILEINIC(LC3D1E)
C******
C
C**   INCLUDE FILES:
      INCLUDE 'a_params.com'
      INCLUDE 'a_i1desc.com'
      INCLUDE 'a_i2desc.com'
      INCLUDE 'a_iopts.com'
      INCLUDE 'a_filfmt.com'
      INCLUDE 'a_units.com'
      INCLUDE 'a_cmmns.com'
C
      INCLUDE 'DIMS3.EXT'
C      INCLUDE 'CONST3.EXT'
      INCLUDE 'PARMS3.EXT'
      INCLUDE 'FDESC3.EXT'
      INCLUDE 'IODECL3.EXT'
      INCLUDE 'RADM.EXT'

      REAL         BUFIC(NCOLS, NROWS, NLAYS, NSPCS)
      integer       ICJDATE, ICJTIME
      integer       STDATE, STTIME
      CHARACTER*16  FNAME, PROGNAME
      DATA          FNAME  / 'INIT_CONC_1     ' /
      DATA          PROGNAME  / 'TRANCONC        ' /
      integer       ISTEP, NSTEP

C
      call mapping
C******
C**        input new hour of chemistry data
C******
C
C             --- open IC file and read initial
C                  time step header -- file type depends on value of
      CALL FILEINI1
      CALL FILEINI2
c
C     open netCDF IC file
c
        read(I1DATE,888) ICJDATE
        read(I1TIME,889) ICJTIME
        write(*,*)' ICJDATE ', ICJDATE, ' ICJTIME ', ICJTIME
        STDATE = 1900000+ICJDATE               ! starting date YYYYDDD
        STTIME = ICJTIME                       ! starting time HHMMSS
        write(*,*)' STDATE ', STDATE, ' STTIME ', STTIME
        FTYPE3D = GRDDED3
        SDATE3D = STDATE
        STIME3D = STTIME
        TSTEP3D = 10000                  ! hourly data
        NVARS3D = NSPCS
        NCOLS3D = NCOLS
        NROWS3D = NROWS
        NLAYS3D = NLAYS
        NTHIK3D =     1
        GDTYP3D = LAMGRD3
        P_ALP3D =    30.0
        P_BET3D =    60.0
        P_GAM3D =  -120.75
        XCENT3D =  -120.75
        YCENT3D =    36.75
        XORIG3D =  -192.0 * 1000.	! in meters  (offset 16)
        YORIG3D =  -234.0 * 1000.	! in meters  (offset 19.5)
        XCELL3D =    12.0 * 1000.	! in meters
        YCELL3D =    12.0 * 1000.	! in meters
        GDNAM3D =  GNAMEX


        FDESC3D( 1 ) =
     &       'Translated from SAQM data provided by CARB'
 

        DO 33 ISPC = 1, NSPCS
cj
             VTYPE3D(ISPC) = M3REAL
cj
            VNAME3D( ISPC ) =  SPCNAMES( ISPC )
            UNITS3D( ISPC ) =  'ppm'
C            UNITS3D( ISPC ) =  'mu g / m**3'
            VDESC3D( ISPC ) =  'Variable ' // SPCNAMES( ISPC ) //' '
C     &                     'Translated from SAQM data provided by CARB'
33      CONTINUE

c
c**  FILE IC
c
cj        IF ( .NOT. CREATE3( FNAME, PROGNAME ) ) THEN
        IF ( .NOT. OPEN3( FNAME, FSUNKN3, PROGNAME ) ) THEN
           CALL M3ERR( 'TRANCONC', STDATE, STTIME,
     &     'Could not open nor create ' // FNAME // ' file', .TRUE. )
        END IF
c

      call filbuf(BUFIC)       ! first hour

c
        IF (.NOT.  WRITE3(FNAME, 'ALL', STDATE, STTIME, BUFIC) ) THEN
           CALL M3ERR( 'TRANCONC', STDATE, STTIME,
     &     'Could not write ' // FNAME // ' file', .TRUE. )
        END IF

        call NEXTIME(STDATE, STTIME,10000)

      read(*,*) NSTEP
      if (NSTEP .le. 1) goto 7777
      do 789 ISTEP = 2, NSTEP

        CALL FILEINI1
        CALL FILEINI2

      call filbuf(BUFIC)       ! first hour

c
        IF (.NOT.  WRITE3(FNAME, 'ALL', STDATE, STTIME, BUFIC) ) THEN
           CALL M3ERR( 'TRANCONC', STDATE, STTIME,
     &     'Could not write ' // FNAME // ' file', .TRUE. )
        END IF

        call NEXTIME(STDATE, STTIME,10000)

 789    CONTINUE

7777    IF ( SHUT3() ) THEN
            WRITE ( *,92999 )
     &  '>>---->  Program  TRANCONC  completed successfully  <----<< '
        ELSE
            WRITE ( *,91000 )
     &      'FATAL ERROR shutting down Models-3 I/O'
        END IF

888     format(I5)
889     format(I6)
92999   FORMAT ( //5X, A, // )

91000   FORMAT ( //5X , '*** ERROR ABORT in program TRANCONC ***',
     &            /5X , A , // )        !  generic error message format
C
      STOP
      END
